/*
*   Copyright (C) 2007 Kevin Krammer <kevin.krammer@gmx.at>
*
*   Permission is hereby granted, free of charge, to any person obtaining a
*   copy of this software and associated documentation files (the "Software"),
*   to deal in the Software without restriction, including without limitation
*   the rights to use, copy, modify, merge, publish, distribute, sublicense,
*   and/or sell copies of the Software, and to permit persons to whom the
*   Software is furnished to do so, subject to the following conditions:
*
*   The above copyright notice and this permission notice shall be included
*   in all copies or substantial portions of the Software.
*
*   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
*   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
*   THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
*   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
*   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
*   OTHER DEALINGS IN THE SOFTWARE.
*/

// Qt includes
#include <qdom.h>
#include <qfile.h>

// local includes
#include "classgen.h"
#include "methodgen.h"

int main(int argc, char** argv)
{
    if (argc < 2)
    {
        qFatal("Missing input file: dbusxml2qt3 <introspection-file>"
               " [NodeClassName]");
    }
    else if (argc > 3)
    {
        qFatal("Too many arguments: dbusxml2qt3 <introspection-file>"
               " [NodeClassName]");
    }

    QString fileName = QString::fromLocal8Bit(argv[1]);

    QFile file(fileName);
    if (!file.exists())
    {
        qFatal("File '%s' does not exist", argv[1]);
    }

    if (!file.open(IO_ReadOnly))
    {
        qFatal("File '%s' can not be read", argv[1]);
    }

    QDomDocument document;

    if (!document.setContent(&file))
    {
        file.close();

        qFatal("Content of file '%s' not parsable", argv[1]);
    }

    file.close();

    QDomElement rootElement = document.documentElement();
    if (rootElement.isNull() || rootElement.tagName() != "node")
    {
        qFatal("File '%s' is not a D-Bus introspection data file", argv[1]);
    }

    QString nodeClassName;
    if (argc == 3) nodeClassName = QString::fromLocal8Bit(argv[2]);

    QValueList<Class> interfaces;
    bool hasIntrospectable = false;

    QDomNode child = rootElement.firstChild();
    for (; !child.isNull(); child = child.nextSibling())
    {
        if (!child.isElement()) continue;

        QDomElement element = child.toElement();

        if (element.tagName() == "interface")
        {
            if (!element.attribute("name").isEmpty())
            {
                Class classData;
                if (ClassGenerator::extractClass(element, classData))
                {
                    ClassGenerator::generateInterface(classData);
                    ClassGenerator::generateProxy(classData);

                    if (classData.dbusName == "org.freedesktop.DBus.Introspectable")
                        hasIntrospectable = true;
                    else
                        interfaces << classData;
                }
            }
        }
    }

    if (!nodeClassName.isEmpty() && !interfaces.isEmpty())
    {
        if (!hasIntrospectable)
        {
            qDebug("Generating org.freedesktop.DBus.Introspectable on demand");

            Class classData;
            classData.name = "Introspectable";
            classData.dbusName = "org.freedesktop.DBus.Introspectable";

            classData.namespaces << "org" << "freedesktop" << "DBus";

            Method method;
            method.name = "Introspect";
            method.noReply = false;

            Argument argument;
            argument.name = "data";
            argument.direction = Argument::Out;
            argument.signature = "QString";
            argument.accessor = "String";
            argument.isPrimitive = false;
            argument.dbusSignature = "s";

            argument.forwardDeclarations << "class QString";
            argument.sourceIncludes["Qt"].append("<qstring.h>");

            method.arguments << argument;
            classData.methods << method;

            ClassGenerator::generateInterface(classData);
        }

        qDebug("Generating node %s", nodeClassName.local8Bit().data());

        QStringList nameParts = QStringList::split("::", nodeClassName);

        Class classData;
        classData.name = nameParts.back();

        nameParts.pop_back();
        classData.namespaces = nameParts;

        ClassGenerator::generateNode(classData, interfaces);
    }

    return 0;
}

// End of File
